import express, { type Application } from "express";
import { readdirSync } from "node:fs";
import { join } from "node:path";
import { Method } from "../interfaces";

export class WebServer {
  private app: Application;
  constructor(port: number) {
    this.app = express();

    this.app.listen(port, () => {
      console.log(`App Listening on port ${port} (http://localhost:${port}/)`);
    });
  }

  init() {
    const savedRoutes: any[] = [];
    const foundRoutes: any[] = [];

    const routes_dir = join(__dirname, "..", "routes");
    const routes_methods = readdirSync(routes_dir);
    if (!routes_methods || !Array.isArray(routes_methods) || routes_methods.length <= 0)
      return;
    console.log(routes_methods)
    routes_methods.forEach((method: string) => {
      method = method.toLowerCase();
      if (!(Object.values(Method) as string[]).includes(method))
        return;
      const method_path = join(routes_dir, method.toUpperCase());
      const routes = readdirSync(method_path).filter(file => file.endsWith(".ts") || file.endsWith(".js"));
      foundRoutes.push(...routes);
      routes.forEach(rt => {
        const { route } = require(join(method_path, rt));
        const { description, run } = route;
        this.app[method as Method](description["route"], ...(description["middleware"] ?? []), run);
        savedRoutes.push(description["route"]);
      });
    });
    console.log(`Successfully Loaded ${savedRoutes.length}/${foundRoutes.length} route${savedRoutes.length !== 1 ? "s" : ""}`)
  }
}

